@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2010 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -56,6 +56,7 @@ from Mailman.Mailbox import ArchiverMailbox
 
 # Set up i18n.  Assume the current language has already been set in the caller.
 _ = i18n._
+C_ = i18n.C_
 
 gzip = None
 if mm_cfg.GZIP_ARCHIVE_TXT_FILES:
@@ -226,9 +227,9 @@ def quick_maketext(templatefile, dict=None, lang=None, mlist=None):
                                     Utils.GetCharSet(lang),
                                     'replace')
                 text = sdict.interpolate(utemplate)
-        except (TypeError, ValueError):
+        except (TypeError, ValueError), e:
             # The template is really screwed up
-            pass
+            syslog('error', 'broken template: %s\n%s', filepath, e)
     # Make sure the text is in the given character set, or html-ify any bogus
     # characters.
     return Utils.uncanonstr(text, lang)
@@ -271,6 +272,8 @@ class Article(pipermail.Article):
             if result:
                 i = result.end(0)
                 self.subject = self.subject[i:]
+                if self.subject == '':
+                    self.subject = _('No subject')
             else:
                 i = -1
         # Useful to keep around
@@ -471,6 +474,7 @@ class Article(pipermail.Article):
             d["email_html"] = self.quote(self.email)
             d["title"] = self.quote(self.subject)
             d["subject_html"] = self.quote(self.subject)
+            d["message_id"] = self.quote(self._message_id)
             # TK: These two _url variables are used to compose a response
             # from the archive web page.  So, ...
             d["subject_url"] = url_quote('Re: ' + self.subject)
@@ -506,7 +510,7 @@ class Article(pipermail.Article):
             subject = self._get_subject_enc(self.prev)
             prev = ('<LINK REL="Previous"  HREF="%s">'
                     % (url_quote(self.prev.filename)))
-            prev_wsubj = ('<LI>' + _('Previous message:') +
+            prev_wsubj = ('<LI>' + _('Previous message (by thread):') +
                           ' <A HREF="%s">%s\n</A></li>'
                           % (url_quote(self.prev.filename),
                              self.quote(subject)))
@@ -528,7 +532,7 @@ class Article(pipermail.Article):
             subject = self._get_subject_enc(self.next)
             next = ('<LINK REL="Next"  HREF="%s">'
                     % (url_quote(self.next.filename)))
-            next_wsubj = ('<LI>' + _('Next message:') +
+            next_wsubj = ('<LI>' + _('Next message (by thread):') +
                           ' <A HREF="%s">%s\n</A></li>'
                           % (url_quote(self.next.filename),
                              self.quote(subject)))
@@ -909,7 +913,7 @@ class HyperArchive(pipermail.T):
         A string can be returned if the list only contains one entry,
         and the empty list is legal."""
         res = self.dateToVolName(float(article.date))
-        self.message(_("figuring article archives\n"))
+        self.message(C_("figuring article archives\n"))
         self.message(res + "\n")
         return res
 
@@ -1047,7 +1051,7 @@ class HyperArchive(pipermail.T):
         self.depth=0
         print self.html_head()
         if not self.THREADLAZY and self.type=='Thread':
-            self.message(_("Computing threaded index\n"))
+            self.message(C_("Computing threaded index\n"))
             self.updateThreadedIndex()
 
     def write_index_footer(self):
@@ -1312,14 +1316,14 @@ class HyperArchive(pipermail.T):
     def update_article(self, arcdir, article, prev, next):
         seq = article.sequence
         filename = os.path.join(arcdir, article.filename)
-        self.message(_('Updating HTML for article %(seq)s'))
+        self.message(C_('Updating HTML for article %(seq)s'))
         try:
             f = open(filename)
             article.loadbody_fromHTML(f)
             f.close()
         except IOError, e:
             if e.errno <> errno.ENOENT: raise
-            self.message(_('article file %(filename)s is missing!'))
+            self.message(C_('article file %(filename)s is missing!'))
         article.prev = prev
         article.next = next
         omask = os.umask(002)
