@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2006 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,16 +24,15 @@ import sys
 import os
 import re
 import cgi
-import sha
 import urllib
 import signal
 from types import *
-from string import lowercase, digits
 
 from email.Utils import unquote, parseaddr, formataddr
 
 from Mailman import mm_cfg
 from Mailman import Utils
+from Mailman import Message
 from Mailman import MailList
 from Mailman import Errors
 from Mailman import MemberAdaptor
@@ -42,10 +41,14 @@ from Mailman.UserDesc import UserDesc
 from Mailman.htmlformat import *
 from Mailman.Cgi import Auth
 from Mailman.Logging.Syslog import syslog
+from Mailman.Utils import sha_new
+from Mailman.CSRFcheck import csrf_check
 
 # Set up i18n
 _ = i18n._
 i18n.set_language(mm_cfg.DEFAULT_SERVER_LANGUAGE)
+def D_(s):
+    return s
 
 NL = '\n'
 OPTCOLUMNS = 11
@@ -56,6 +59,8 @@ except NameError:
     True = 1
     False = 0
 
+AUTH_CONTEXTS = (mm_cfg.AuthListAdmin, mm_cfg.AuthSiteAdmin)
+
 
 
 def main():
@@ -72,22 +77,58 @@ def main():
     except Errors.MMListError, e:
         # Avoid cross-site scripting attacks
         safelistname = Utils.websafe(listname)
+        # Send this with a 404 status.
+        print 'Status: 404 Not Found'
         admin_overview(_('No such list <em>%(safelistname)s</em>'))
-        syslog('error', 'admin.py access for non-existent list: %s',
-               listname)
+        syslog('error', 'admin: No such list "%s": %s\n',
+               listname, e)
         return
     # Now that we know what list has been requested, all subsequent admin
     # pages are shown in that list's preferred language.
     i18n.set_language(mlist.preferred_language)
     # If the user is not authenticated, we're done.
     cgidata = cgi.FieldStorage(keep_blank_values=1)
+    try:
+        cgidata.getfirst('csrf_token', '')
+    except TypeError:
+        # Someone crafted a POST with a bad Content-Type:.
+        doc = Document()
+        doc.set_language(mm_cfg.DEFAULT_SERVER_LANGUAGE)
+        doc.AddItem(Header(2, _("Error")))
+        doc.AddItem(Bold(_('Invalid options to CGI script.')))
+        # Send this with a 400 status.
+        print 'Status: 400 Bad Request'
+        print doc.Format()
+        return
+
+    # CSRF check
+    safe_params = ['VARHELP', 'adminpw', 'admlogin',
+                   'letter', 'chunk', 'findmember',
+                   'legend']
+    params = cgidata.keys()
+    if set(params) - set(safe_params):
+        csrf_checked = csrf_check(mlist, cgidata.getfirst('csrf_token'),
+                                  'admin')
+    else:
+        csrf_checked = True
+    # if password is present, void cookie to force password authentication.
+    if cgidata.getfirst('adminpw'):
+        os.environ['HTTP_COOKIE'] = ''
+        csrf_checked = True
 
     if not mlist.WebAuthenticate((mm_cfg.AuthListAdmin,
                                   mm_cfg.AuthSiteAdmin),
-                                 cgidata.getvalue('adminpw', '')):
+                                 cgidata.getfirst('adminpw', '')):
         if cgidata.has_key('adminpw'):
             # This is a re-authorization attempt
             msg = Bold(FontSize('+1', _('Authorization failed.'))).Format()
+            remote = os.environ.get('HTTP_FORWARDED_FOR',
+                     os.environ.get('HTTP_X_FORWARDED_FOR',
+                     os.environ.get('REMOTE_ADDR',
+                                    'unidentified origin')))
+            syslog('security',
+                   'Authorization failed (admin): list=%s: remote=%s',
+                   listname, remote)
         else:
             msg = ''
         Auth.loginpage(mlist, 'admin', msg=msg)
@@ -106,6 +147,9 @@ def main():
 
     # Is this a log-out request?
     if category == 'logout':
+        # site-wide admin should also be able to logout.
+        if mlist.AuthContextInfo(mm_cfg.AuthSiteAdmin)[0] == 'site':
+            print mlist.ZapCookie(mm_cfg.AuthSiteAdmin)
         print mlist.ZapCookie(mm_cfg.AuthListAdmin)
         Auth.loginpage(mlist, 'admin', frontpage=1)
         return
@@ -121,7 +165,7 @@ def main():
     if qsenviron:
         parsedqs = cgi.parse_qs(qsenviron)
     if cgidata.has_key('VARHELP'):
-        varhelp = cgidata.getvalue('VARHELP')
+        varhelp = cgidata.getfirst('VARHELP')
     elif parsedqs:
         # POST methods, even if their actions have a query string, don't get
         # put into FieldStorage's keys :-(
@@ -170,8 +214,12 @@ def main():
         signal.signal(signal.SIGTERM, sigterm_handler)
 
         if cgidata.keys():
-            # There are options to change
-            change_options(mlist, category, subcat, cgidata, doc)
+            if csrf_checked:
+                # There are options to change
+                change_options(mlist, category, subcat, cgidata, doc)
+            else:
+                doc.addError(
+                  _('The form lifetime has expired. (request forgery check)'))
             # Let the list sanity check the changed values
             mlist.CheckValues()
         # Additional sanity checks
@@ -183,16 +231,20 @@ def main():
                 non-digest delivery or your mailing list will basically be
                 unusable.'''), tag=_('Warning: '))
 
-        if not mlist.digestable and mlist.getDigestMemberKeys():
+        dm = mlist.getDigestMemberKeys()
+        if not mlist.digestable and dm:
             doc.addError(
                 _('''You have digest members, but digests are turned
-                off. Those people will not receive mail.'''),
+                off. Those people will not receive mail.
+                Affected member(s) %(dm)r.'''),
                 tag=_('Warning: '))
-        if not mlist.nondigestable and mlist.getRegularMemberKeys():
+        rm = mlist.getRegularMemberKeys()
+        if not mlist.nondigestable and rm:
             doc.addError(
                 _('''You have regular list members but non-digestified mail is
-                turned off.  They will receive mail until you fix this
-                problem.'''), tag=_('Warning: '))
+                turned off.  They will receive non-digestified mail until you
+                fix this problem. Affected member(s) %(rm)r.'''),
+                tag=_('Warning: '))
         # Glom up the results page and print it out
         show_results(mlist, doc, category, subcat, cgidata)
         print doc.Format()
@@ -230,21 +282,26 @@ def admin_overview(msg=''):
     listnames.sort()
 
     for name in listnames:
-        mlist = MailList.MailList(name, lock=0)
+        try:
+            mlist = MailList.MailList(name, lock=0)
+        except Errors.MMUnknownListError:
+            # The list could have been deleted by another process.
+            continue
         if mlist.advertised:
-            if mm_cfg.VIRTUAL_HOST_OVERVIEW and \
-                   mlist.web_page_url.find(hostname) == -1:
+            if mm_cfg.VIRTUAL_HOST_OVERVIEW and (
+                   mlist.web_page_url.find('/%s/' % hostname) == -1 and
+                   mlist.web_page_url.find('/%s:' % hostname) == -1):
                 # List is for different identity of this host - skip it.
                 continue
             else:
                 advertised.append((mlist.GetScriptURL('admin'),
                                    mlist.real_name,
-                                   mlist.description))
+                                   Utils.websafe(mlist.GetDescription())))
     # Greeting depends on whether there was an error or not
     if msg:
         greeting = FontAttr(msg, color="ff5060", size="+1")
     else:
-        greeting = _("Welcome!")
+        greeting = FontAttr(_('Welcome!'), size='+2')
 
     welcome = []
     mailmanlink = Link(mm_cfg.MAILMAN_URL, _('Mailman')).Format()
@@ -319,10 +376,11 @@ def option_help(mlist, varhelp):
         elif len(reflist) == 3:
             category, subcat, varname = reflist
         options = mlist.GetConfigInfo(category, subcat)
-        for i in options:
-            if i and i[0] == varname:
-                item = i
-                break
+        if options:
+            for i in options:
+                if i and i[0] == varname:
+                    item = i
+                    break
     # Print an error message if we couldn't find a valid one
     if not item:
         bad = _('No valid variable name found.')
@@ -352,7 +410,7 @@ def option_help(mlist, varhelp):
         url = '%s/%s/%s' % (mlist.GetScriptURL('admin'), category, subcat)
     else:
         url = '%s/%s' % (mlist.GetScriptURL('admin'), category)
-    form = Form(url)
+    form = Form(url, mlist=mlist, contexts=AUTH_CONTEXTS)
     valtab = Table(cellspacing=3, cellpadding=4, width='100%')
     add_options_table_item(mlist, category, subcat, valtab, item, detailsp=0)
     form.AddItem(valtab)
@@ -398,9 +456,10 @@ def show_results(mlist, doc, category, subcat, cgidata):
         encoding = 'multipart/form-data'
     if subcat:
         form = Form('%s/%s/%s' % (adminurl, category, subcat),
-                    encoding=encoding)
+                    encoding=encoding, mlist=mlist, contexts=AUTH_CONTEXTS)
     else:
-        form = Form('%s/%s' % (adminurl, category), encoding=encoding)
+        form = Form('%s/%s' % (adminurl, category),
+                    encoding=encoding, mlist=mlist, contexts=AUTH_CONTEXTS)
     # This holds the two columns of links
     linktable = Table(valign='top', width='100%')
     linktable.AddRow([Center(Bold(_("Configuration Categories"))),
@@ -491,7 +550,7 @@ def show_results(mlist, doc, category, subcat, cgidata):
     if category == 'members':
         # Figure out which subcategory we should display
         subcat = Utils.GetPathPieces()[-1]
-        if subcat not in ('list', 'add', 'remove'):
+        if subcat not in ('list', 'add', 'remove', 'change', 'sync'):
             subcat = 'list'
         # Add member category specific tables
         form.AddItem(membership_options(mlist, subcat, cgidata, doc, form))
@@ -845,6 +904,20 @@ def membership_options(mlist, subcat, cgidata, doc, form):
         container.AddItem(header)
         mass_remove(mlist, container)
         return container
+    if subcat == 'change':
+        header.AddRow([Center(Header(2, _('Address Change')))])
+        header.AddCellInfo(header.GetCurrentRowIndex(), 0, colspan=2,
+                           bgcolor=mm_cfg.WEB_HEADER_COLOR)
+        container.AddItem(header)
+        address_change(mlist, container)
+        return container
+    if subcat == 'sync':
+        header.AddRow([Center(Header(2, _('Sync Membership List')))])
+        header.AddCellInfo(header.GetCurrentRowIndex(), 0, colspan=2,
+                           bgcolor=mm_cfg.WEB_HEADER_COLOR)
+        container.AddItem(header)
+        mass_sync(mlist, container)
+        return container
     # Otherwise...
     header.AddRow([Center(Header(2, _('Membership List')))])
     header.AddCellInfo(header.GetCurrentRowIndex(), 0, colspan=2,
@@ -852,11 +925,12 @@ def membership_options(mlist, subcat, cgidata, doc, form):
     container.AddItem(header)
     # Add a "search for member" button
     table = Table(width='100%')
-    link = Link('http://www.python.org/doc/current/lib/re-syntax.html',
+    link = Link('https://docs.python.org/2/library/re.html'
+                '#regular-expression-syntax',
                 _('(help)')).Format()
     table.AddRow([Label(_('Find member %(link)s:')),
                   TextBox('findmember',
-                          value=cgidata.getvalue('findmember', '')),
+                          value=cgidata.getfirst('findmember', '')),
                   SubmitButton('findmember_btn', _('Search...'))])
     container.AddItem(table)
     container.AddItem('<hr><p>')
@@ -869,7 +943,16 @@ def membership_options(mlist, subcat, cgidata, doc, form):
     all = [_m.encode() for _m in mlist.getMembers()]
     all.sort(lambda x, y: cmp(x.lower(), y.lower()))
     # See if the query has a regular expression
-    regexp = cgidata.getvalue('findmember', '').strip()
+    regexp = cgidata.getfirst('findmember', '').strip()
+    try:
+        regexp = regexp.decode(Utils.GetCharSet(mlist.preferred_language))
+    except UnicodeDecodeError:
+        # This is probably a non-ascii character and an English language
+        # (ascii) list.  Even if we didn't throw the UnicodeDecodeError,
+        # the input may have contained mnemonic or numeric HTML entites mixed
+        # with other characters.  Trying to grok the real meaning out of that
+        # is complex and error prone, so we don't try.
+        pass
     if regexp:
         try:
             cre = re.compile(regexp, re.IGNORECASE)
@@ -900,12 +983,10 @@ def membership_options(mlist, subcat, cgidata, doc, form):
         qsenviron = os.environ.get('QUERY_STRING')
         if qsenviron:
             qs = cgi.parse_qs(qsenviron)
-            bucket = qs.get('letter', 'a')[0].lower()
-            if bucket not in digits + lowercase:
-                bucket = None
+            bucket = qs.get('letter', '0')[0].lower()
+        keys = buckets.keys()
+        keys.sort()
         if not bucket or not buckets.has_key(bucket):
-            keys = buckets.keys()
-            keys.sort()
             bucket = keys[0]
         members = buckets[bucket]
         action = adminurl + '/members?letter=%s' % bucket
@@ -941,10 +1022,14 @@ def membership_options(mlist, subcat, cgidata, doc, form):
     # Add the alphabetical links
     if bucket:
         cells = []
-        for letter in digits + lowercase:
-            if not buckets.get(letter):
-                continue
-            url = adminurl + '/members?letter=%s' % letter
+        for letter in keys:
+            findfrag = ''
+            if regexp:
+                findfrag = '&findmember=' + urllib.quote(regexp)
+            url = adminurl + '/members?letter=' + letter + findfrag
+            if isinstance(url, unicode):
+                url = url.encode(Utils.GetCharSet(mlist.preferred_language),
+                                 errors='ignore')
             if letter == bucket:
                 show = Bold('[%s]' % letter.upper()).Format()
             else:
@@ -981,15 +1066,17 @@ def membership_options(mlist, subcat, cgidata, doc, form):
                   }
     # Now populate the rows
     for addr in members:
+        qaddr = urllib.quote(addr)
         link = Link(mlist.GetOptionsURL(addr, obscure=1),
                     mlist.getMemberCPAddress(addr))
         fullname = Utils.uncanonstr(mlist.getMemberName(addr),
                                     mlist.preferred_language)
-        name = TextBox(addr + '_realname', fullname, size=longest).Format()
-        cells = [Center(CheckBox(addr + '_unsub', 'off', 0).Format()),
+        name = TextBox(qaddr + '_realname', fullname, size=longest).Format()
+        cells = [Center(CheckBox(qaddr + '_unsub', 'off', 0).Format()
+                        + '<div class="hidden">' + _('unsub') + '</div>'),
                  link.Format() + '<br>' +
                  name +
-                 Hidden('user', urllib.quote(addr)).Format(),
+                 Hidden('user', qaddr).Format(),
                  ]
         # Do the `mod' option
         if mlist.getMemberOption(addr, mm_cfg.Moderate):
@@ -998,10 +1085,13 @@ def membership_options(mlist, subcat, cgidata, doc, form):
         else:
             value = 'off'
             checked = 0
-        box = CheckBox('%s_mod' % addr, value, checked)
-        cells.append(Center(box).Format())
+        box = CheckBox('%s_mod' % qaddr, value, checked)
+        cells.append(Center(box.Format()
+            + '<div class="hidden">' + _('mod') + '</div>'))
+        # Kluge, get these translated.
+        (_('hide'), _('nomail'), _('ack'), _('notmetoo'), _('nodupes'))
         for opt in ('hide', 'nomail', 'ack', 'notmetoo', 'nodupes'):
-            extra = ''
+            extra = '<div class="hidden">' + _(opt) + '</div>'
             if opt == 'nomail':
                 status = mlist.getDeliveryStatus(addr)
                 if status == MemberAdaptor.ENABLED:
@@ -1010,30 +1100,35 @@ def membership_options(mlist, subcat, cgidata, doc, form):
                 else:
                     value = 'on'
                     checked = 1
-                    extra = '[%s]' % ds_abbrevs[status]
+                    extra = '[%s]' % ds_abbrevs[status] + extra
             elif mlist.getMemberOption(addr, mm_cfg.OPTINFO[opt]):
                 value = 'on'
                 checked = 1
             else:
                 value = 'off'
                 checked = 0
-            box = CheckBox('%s_%s' % (addr, opt), value, checked)
+            box = CheckBox('%s_%s' % (qaddr, opt), value, checked)
             cells.append(Center(box.Format() + extra))
         # This code is less efficient than the original which did a has_key on
         # the underlying dictionary attribute.  This version is slower and
         # less memory efficient.  It points to a new MemberAdaptor interface
         # method.
+        extra = '<div class="hidden">' + _('digest') + '</div>'
         if addr in mlist.getRegularMemberKeys():
-            cells.append(Center(CheckBox(addr + '_digest', 'off', 0).Format()))
+            cells.append(Center(CheckBox(qaddr + '_digest', 'off', 0).Format()
+                                + extra))
         else:
-            cells.append(Center(CheckBox(addr + '_digest', 'on', 1).Format()))
+            cells.append(Center(CheckBox(qaddr + '_digest', 'on', 1).Format()
+                                + extra))
         if mlist.getMemberOption(addr, mm_cfg.OPTINFO['plain']):
             value = 'on'
             checked = 1
         else:
             value = 'off'
             checked = 0
-        cells.append(Center(CheckBox('%s_plain' % addr, value, checked)))
+        cells.append(Center(CheckBox(
+                            '%s_plain' % qaddr, value, checked).Format()
+                            + '<div class="hidden">' + _('plain') + '</div>'))
         # User's preferred language
         langpref = mlist.getMemberLanguage(addr)
         langs = mlist.GetAvailableLanguages()
@@ -1042,7 +1137,7 @@ def membership_options(mlist, subcat, cgidata, doc, form):
             selected = langs.index(langpref)
         except ValueError:
             selected = 0
-        cells.append(Center(SelectOptions(addr + '_language', langs,
+        cells.append(Center(SelectOptions(qaddr + '_language', langs,
                                           langdescs, selected)).Format())
         usertable.AddRow(cells)
     # Add the usertable and a legend
@@ -1119,7 +1214,12 @@ def membership_options(mlist, subcat, cgidata, doc, form):
                 continue
             start = chunkmembers[i*chunksz]
             end = chunkmembers[min((i+1)*chunksz, last)-1]
-            link = Link(url + 'chunk=%d' % i, _('from %(start)s to %(end)s'))
+            thisurl = url + 'chunk=%d' % i + findfrag
+            if isinstance(thisurl, unicode):
+                thisurl = thisurl.encode(
+                                 Utils.GetCharSet(mlist.preferred_language),
+                                 errors='ignore')
+            link = Link(thisurl, _('from %(start)s to %(end)s'))
             buttons.append(link)
         buttons = UnorderedList(*buttons)
         container.AddItem(footer + buttons.Format() + '<p>')
@@ -1135,12 +1235,13 @@ def mass_subscribe(mlist, container):
         Label(_('Subscribe these users now or invite them?')),
         RadioButtonArray('subscribe_or_invite',
                          (_('Subscribe'), _('Invite')),
-                         0, values=(0, 1))
+                         mm_cfg.DEFAULT_SUBSCRIBE_OR_INVITE,
+                         values=(0, 1))
         ])
     table.AddCellInfo(table.GetCurrentRowIndex(), 0, bgcolor=GREY)
     table.AddCellInfo(table.GetCurrentRowIndex(), 1, bgcolor=GREY)
     table.AddRow([
-        Label(_('Send welcome messages to new subscribees?')),
+        Label(_('Send welcome messages to new subscribers?')),
         RadioButtonArray('send_welcome_msg_to_this_batch',
                          (_('No'), _('Yes')),
                          mlist.send_welcome_msg,
@@ -1208,6 +1309,52 @@ def mass_remove(mlist, container):
     container.AddItem(Center(table))
 
 
+
+def address_change(mlist, container):
+    # ADDRESS CHANGE
+    GREY = mm_cfg.WEB_ADMINITEM_COLOR
+    table = Table(width='90%')
+    table.AddRow([Italic(_("""To change a list member's address, enter the
+    member's current and new addresses below. Use the check boxes to send
+    notice of the change to the old and/or new address(es)."""))])
+    table.AddCellInfo(table.GetCurrentRowIndex(), 0, colspan=3)
+    table.AddRow([
+        Label(_("Member's current address")),
+        TextBox(name='change_from'),
+        CheckBox('notice_old', 'yes', 0).Format() +
+            '&nbsp;' +
+            _('Send notice')
+        ])
+    table.AddCellInfo(table.GetCurrentRowIndex(), 0, bgcolor=GREY)
+    table.AddCellInfo(table.GetCurrentRowIndex(), 1, bgcolor=GREY)
+    table.AddCellInfo(table.GetCurrentRowIndex(), 2, bgcolor=GREY)
+    table.AddRow([
+        Label(_('Address to change to')),
+        TextBox(name='change_to'),
+        CheckBox('notice_new', 'yes', 0).Format() +
+            '&nbsp;' +
+            _('Send notice')
+        ])
+    table.AddCellInfo(table.GetCurrentRowIndex(), 0, bgcolor=GREY)
+    table.AddCellInfo(table.GetCurrentRowIndex(), 1, bgcolor=GREY)
+    table.AddCellInfo(table.GetCurrentRowIndex(), 2, bgcolor=GREY)
+    container.AddItem(Center(table))
+
+
+
+def mass_sync(mlist, container):
+    # MASS SYNC
+    table = Table(width='90%')
+    table.AddRow([Italic(_('Enter one address per line below...'))])
+    table.AddCellInfo(table.GetCurrentRowIndex(), 0, colspan=2)
+    table.AddRow([Center(TextArea(name='memberlist',
+                                  rows=10, cols='70%', wrap=None))])
+    table.AddCellInfo(table.GetCurrentRowIndex(), 0, colspan=2)
+    table.AddRow([Italic(Label(_('...or specify a file to upload:'))),
+                  FileUpload('memberlist_upload', cols='50')])
+    container.AddItem(Center(table))
+
+
 
 def password_inputs(mlist):
     adminurl = mlist.GetScriptURL('admin', absolute=1)
@@ -1247,6 +1394,22 @@ and also provide the email addresses of the list moderators in the
                    PasswordBox('confirmmodpw', size=20)])
     # Add these tables to the overall password table
     table.AddRow([atable, mtable])
+    table.AddRow([_("""\
+In addition to the above passwords you may specify a password for
+pre-approving posts to the list. Either of the above two passwords can
+be used in an Approved: header or first body line pseudo-header to
+pre-approve a post that would otherwise be held for moderation. In
+addition, the password below, if set, can be used for that purpose and
+no other.""")])
+    table.AddCellInfo(table.GetCurrentRowIndex(), 0, colspan=2)
+    # Set up the post password table
+    ptable = Table(border=0, cellspacing=3, cellpadding=4,
+                   bgcolor=mm_cfg.WEB_ADMINPW_COLOR)
+    ptable.AddRow([Label(_('Enter new poster password:')),
+                   PasswordBox('newpostpw', size=20)])
+    ptable.AddRow([Label(_('Confirm poster password:')),
+                   PasswordBox('confirmpostpw', size=20)])
+    table.AddRow([ptable])
     return table
 
 
@@ -1260,29 +1423,41 @@ def submit_button(name='submit'):
 
 
 def change_options(mlist, category, subcat, cgidata, doc):
+    global _
     def safeint(formvar, defaultval=None):
         try:
-            return int(cgidata.getvalue(formvar))
+            return int(cgidata.getfirst(formvar))
         except (ValueError, TypeError):
             return defaultval
     confirmed = 0
     # Handle changes to the list moderator password.  Do this before checking
     # the new admin password, since the latter will force a reauthentication.
-    new = cgidata.getvalue('newmodpw', '').strip()
-    confirm = cgidata.getvalue('confirmmodpw', '').strip()
+    new = cgidata.getfirst('newmodpw', '').strip()
+    confirm = cgidata.getfirst('confirmmodpw', '').strip()
     if new or confirm:
         if new == confirm:
-            mlist.mod_password = sha.new(new).hexdigest()
+            mlist.mod_password = sha_new(new).hexdigest()
             # No re-authentication necessary because the moderator's
             # password doesn't get you into these pages.
         else:
             doc.addError(_('Moderator passwords did not match'))
-    # Handle changes to the list administrator password
-    new = cgidata.getvalue('newpw', '').strip()
-    confirm = cgidata.getvalue('confirmpw', '').strip()
+    # Handle changes to the list poster password.  Do this before checking
+    # the new admin password, since the latter will force a reauthentication.
+    new = cgidata.getfirst('newpostpw', '').strip()
+    confirm = cgidata.getfirst('confirmpostpw', '').strip()
     if new or confirm:
         if new == confirm:
-            mlist.password = sha.new(new).hexdigest()
+            mlist.post_password = sha_new(new).hexdigest()
+            # No re-authentication necessary because the poster's
+            # password doesn't get you into these pages.
+        else:
+            doc.addError(_('Poster passwords did not match'))
+    # Handle changes to the list administrator password
+    new = cgidata.getfirst('newpw', '').strip()
+    confirm = cgidata.getfirst('confirmpw', '').strip()
+    if new or confirm:
+        if new == confirm:
+            mlist.password = sha_new(new).hexdigest()
             # Set new cookie
             print mlist.MakeCookie(mm_cfg.AuthListAdmin)
         else:
@@ -1295,8 +1470,8 @@ def change_options(mlist, category, subcat, cgidata, doc):
         gui.handleForm(mlist, category, subcat, cgidata, doc)
     # mass subscription, removal processing for members category
     subscribers = ''
-    subscribers += cgidata.getvalue('subscribees', '')
-    subscribers += cgidata.getvalue('subscribees_upload', '')
+    subscribers += cgidata.getfirst('subscribees', '')
+    subscribers += cgidata.getfirst('subscribees_upload', '')
     if subscribers:
         entries = filter(None, [n.strip() for n in subscribers.splitlines()])
         send_welcome_msg = safeint('send_welcome_msg_to_this_batch',
@@ -1305,7 +1480,7 @@ def change_options(mlist, category, subcat, cgidata, doc):
                                    mlist.admin_notify_mchanges)
         # Default is to subscribe
         subscribe_or_invite = safeint('subscribe_or_invite', 0)
-        invitation = cgidata.getvalue('invitation', '')
+        invitation = cgidata.getfirst('invitation', '')
         digest = mlist.digest_is_default
         if not mlist.digestable:
             digest = 0
@@ -1332,9 +1507,12 @@ def change_options(mlist, category, subcat, cgidata, doc):
                     else:
                         mlist.InviteNewMember(userdesc, invitation)
                 else:
+                    _ = D_
+                    whence = _('admin mass sub')
+                    _ = i18n._
                     mlist.ApprovedAddMember(userdesc, send_welcome_msg,
                                             send_admin_notif, invitation,
-                                            whence='admin mass sub')
+                                            whence=whence)
             except Errors.MMAlreadyAMember:
                 subscribe_errors.append((safeentry, _('Already a member')))
             except Errors.MMBadEmailError:
@@ -1377,16 +1555,21 @@ def change_options(mlist, category, subcat, cgidata, doc):
         removals += cgidata['unsubscribees_upload'].value
     if removals:
         names = filter(None, [n.strip() for n in removals.splitlines()])
-        send_unsub_notifications = int(
-            cgidata['send_unsub_notifications_to_list_owner'].value)
-        userack = int(
-            cgidata['send_unsub_ack_to_this_batch'].value)
+        send_unsub_notifications = safeint(
+            'send_unsub_notifications_to_list_owner',
+            mlist.admin_notify_mchanges)
+        userack = safeint(
+            'send_unsub_ack_to_this_batch',
+            mlist.send_goodbye_msg)
         unsubscribe_errors = []
         unsubscribe_success = []
         for addr in names:
             try:
+                _ = D_
+                whence = _('admin mass unsub')
+                _ = i18n._
                 mlist.ApprovedDeleteMember(
-                    addr, whence='admin mass unsub',
+                    addr, whence=whence,
                     admin_notif=send_unsub_notifications,
                     userack=userack)
                 unsubscribe_success.append(Utils.websafe(addr))
@@ -1402,13 +1585,163 @@ def change_options(mlist, category, subcat, cgidata, doc):
                 color='#ff0000', size='+2')).Format()))
             doc.AddItem(UnorderedList(*unsubscribe_errors))
             doc.AddItem('<p>')
+    # Address Changes
+    if cgidata.has_key('change_from'):
+        change_from = cgidata.getfirst('change_from', '')
+        change_to = cgidata.getfirst('change_to', '')
+        schange_from = Utils.websafe(change_from)
+        schange_to = Utils.websafe(change_to)
+        success = False
+        msg = None
+        if not (change_from and change_to):
+            msg = _('You must provide both current and new addresses.')
+        elif change_from == change_to:
+            msg = _('Current and new addresses must be different.')
+        elif mlist.isMember(change_to):
+            # ApprovedChangeMemberAddress will just delete the old address
+            # and we don't want that here.
+            msg = _('%(schange_to)s is already a list member.')
+        else:
+            try:
+                Utils.ValidateEmail(change_to)
+            except (Errors.MMBadEmailError, Errors.MMHostileAddress):
+                msg = _('%(schange_to)s is not a valid email address.')
+        if msg:
+            doc.AddItem(Header(3, msg))
+            doc.AddItem('<p>')
+            return
+        try:
+            mlist.ApprovedChangeMemberAddress(change_from, change_to, False)
+        except Errors.NotAMemberError:
+            msg = _('%(schange_from)s is not a member')
+        except Errors.MMAlreadyAMember:
+            msg = _('%(schange_to)s is already a member')
+        except Errors.MembershipIsBanned, pat:
+            spat = Utils.websafe(str(pat))
+            msg = _('%(schange_to)s matches banned pattern %(spat)s')
+        else:
+            msg = _('Address %(schange_from)s changed to %(schange_to)s')
+            success = True
+        doc.AddItem(Header(3, msg))
+        lang = mlist.getMemberLanguage(change_to)
+        otrans = i18n.get_translation()
+        i18n.set_language(lang)
+        list_name = mlist.getListAddress()
+        text = Utils.wrap(_("""The member address %(change_from)s on the
+%(list_name)s list has been changed to %(change_to)s.
+"""))
+        subject = _('%(list_name)s address change notice.')
+        i18n.set_translation(otrans)
+        if success and cgidata.getfirst('notice_old', '') == 'yes':
+            # Send notice to old address.
+            msg = Message.UserNotification(change_from,
+                mlist.GetOwnerEmail(),
+                text=text,
+                subject=subject,
+                lang=lang
+                )
+            msg.send(mlist)
+            doc.AddItem(Header(3, _('Notification sent to %(schange_from)s.')))
+        if success and cgidata.getfirst('notice_new', '') == 'yes':
+            # Send notice to new address.
+            msg = Message.UserNotification(change_to,
+                mlist.GetOwnerEmail(),
+                text=text,
+                subject=subject,
+                lang=lang
+                )
+            msg.send(mlist)
+            doc.AddItem(Header(3, _('Notification sent to %(schange_to)s.')))
+        doc.AddItem('<p>')
+
+    # sync operation
+    memberlist = ''
+    memberlist += cgidata.getvalue('memberlist', '')
+    memberlist += cgidata.getvalue('memberlist_upload', '')
+    if memberlist:
+        # Browsers will convert special characters in the text box to HTML
+        # entities. We need to fix those.
+        def i_to_c(mo):
+            # Convert a matched string of digits to the corresponding unicode.
+            return unichr(int(mo.group(1)))
+        def clean_input(x):
+            # Strip leading/trailing whitespace and convert numeric HTML
+            # entities.
+            return re.sub(r'&#(\d+);', i_to_c, x.strip())
+        entries = filter(None,
+                         [clean_input(n) for n in memberlist.splitlines()])
+        lc_addresses = [parseaddr(x)[1].lower() for x in entries
+                        if parseaddr(x)[1]]
+        subscribe_errors = []
+        subscribe_success = []
+        # First we add all the addresses that should be added to the list.
+        for entry in entries:
+            safeentry = Utils.websafe(entry)
+            fullname, address = parseaddr(entry)
+            if mlist.isMember(address):
+                continue
+            # Canonicalize the full name.
+            fullname = Utils.canonstr(fullname, mlist.preferred_language)
+            userdesc = UserDesc(address, fullname,
+                                Utils.MakeRandomPassword(),
+                                0, mlist.preferred_language)
+            try:
+                # Add a member if not yet member.
+                    mlist.ApprovedAddMember(userdesc, 0, 0, 0,
+                                            whence='admin sync members')
+            except Errors.MMBadEmailError:
+                if userdesc.address == '':
+                    subscribe_errors.append((_('&lt;blank line&gt;'),
+                                             _('Bad/Invalid email address')))
+                else:
+                    subscribe_errors.append((safeentry,
+                                             _('Bad/Invalid email address')))
+            except Errors.MMHostileAddress:
+                subscribe_errors.append(
+                    (safeentry, _('Hostile address (illegal characters)')))
+            except Errors.MembershipIsBanned, pattern:
+                subscribe_errors.append(
+                    (safeentry, _('Banned address (matched %(pattern)s)')))
+            else:
+                member = Utils.uncanonstr(formataddr((fullname, address)))
+                subscribe_success.append(Utils.websafe(member))
+
+        # Then we remove the addresses not in our list.
+        unsubscribe_errors = []
+        unsubscribe_success = []
+
+        for entry in mlist.getMembers():
+            # If an entry is not found in the uploaded "entries" list, then
+            # remove the member.
+            if not(entry in lc_addresses):
+                try:
+                    mlist.ApprovedDeleteMember(entry, 0, 0)
+                except Errors.NotAMemberError:
+                    # This can happen if the address is illegal (i.e. can't be
+                    # parsed by email.Utils.parseaddr()) but for legacy
+                    # reasons is in the database.  Use a lower level remove to
+                    # get rid of this member's entry
+                    mlist.removeMember(entry)
+                else:
+                    unsubscribe_success.append(Utils.websafe(entry))
+
+        if subscribe_success:
+            doc.AddItem(Header(5, _('Successfully subscribed:')))
+            doc.AddItem(UnorderedList(*subscribe_success))
+            doc.AddItem('<p>')
+        if subscribe_errors:
+            doc.AddItem(Header(5, _('Error subscribing:')))
+            items = ['%s -- %s' % (x0, x1) for x0, x1 in subscribe_errors]
+            doc.AddItem(UnorderedList(*items))
+            doc.AddItem('<p>')
+        if unsubscribe_success:
+            doc.AddItem(Header(5, _('Successfully Unsubscribed:')))
+            doc.AddItem(UnorderedList(*unsubscribe_success))
+            doc.AddItem('<p>')
+
     # See if this was a moderation bit operation
     if cgidata.has_key('allmodbit_btn'):
-        val = cgidata.getvalue('allmodbit_val')
-        try:
-            val = int(val)
-        except VallueError:
-            val = None
+        val = safeint('allmodbit_val')
         if val not in (0, 1):
             doc.addError(_('Bad moderation flag value'))
         else:
@@ -1426,9 +1759,13 @@ def change_options(mlist, category, subcat, cgidata, doc):
         errors = []
         removes = []
         for user in users:
-            if cgidata.has_key('%s_unsub' % user):
+            quser = urllib.quote(user)
+            if cgidata.has_key('%s_unsub' % quser):
                 try:
-                    mlist.ApprovedDeleteMember(user, whence='member mgt page')
+                    _ = D_
+                    whence=_('member mgt page')
+                    _ = i18n._
+                    mlist.ApprovedDeleteMember(user, whence=whence)
                     removes.append(user)
                 except Errors.NotAMemberError:
                     errors.append((user, _('Not subscribed')))
@@ -1437,7 +1774,7 @@ def change_options(mlist, category, subcat, cgidata, doc):
                 doc.addError(_('Ignoring changes to deleted member: %(user)s'),
                              tag=_('Warning: '))
                 continue
-            value = cgidata.has_key('%s_digest' % user)
+            value = cgidata.has_key('%s_digest' % quser)
             try:
                 mlist.setMemberOption(user, mm_cfg.Digests, value)
             except (Errors.AlreadyReceivingDigests,
@@ -1447,28 +1784,28 @@ def change_options(mlist, category, subcat, cgidata, doc):
                 # BAW: Hmm...
                 pass
 
-            newname = cgidata.getvalue(user+'_realname', '')
+            newname = cgidata.getfirst(quser+'_realname', '')
             newname = Utils.canonstr(newname, mlist.preferred_language)
             mlist.setMemberName(user, newname)
 
-            newlang = cgidata.getvalue(user+'_language')
+            newlang = cgidata.getfirst(quser+'_language')
             oldlang = mlist.getMemberLanguage(user)
             if Utils.IsLanguage(newlang) and newlang <> oldlang:
                 mlist.setMemberLanguage(user, newlang)
 
-            moderate = not not cgidata.getvalue(user+'_mod')
+            moderate = not not cgidata.getfirst(quser+'_mod')
             mlist.setMemberOption(user, mm_cfg.Moderate, moderate)
 
             # Set the `nomail' flag, but only if the user isn't already
             # disabled (otherwise we might change BYUSER into BYADMIN).
-            if cgidata.has_key('%s_nomail' % user):
+            if cgidata.has_key('%s_nomail' % quser):
                 if mlist.getDeliveryStatus(user) == MemberAdaptor.ENABLED:
                     mlist.setDeliveryStatus(user, MemberAdaptor.BYADMIN)
             else:
                 mlist.setDeliveryStatus(user, MemberAdaptor.ENABLED)
             for opt in ('hide', 'ack', 'notmetoo', 'nodupes', 'plain'):
                 opt_code = mm_cfg.OPTINFO[opt]
-                if cgidata.has_key('%s_%s' % (user, opt)):
+                if cgidata.has_key('%s_%s' % (quser, opt)):
                     mlist.setMemberOption(user, opt_code, 1)
                 else:
                     mlist.setMemberOption(user, opt_code, 0)
