@@ -1648,10 +1648,10 @@ PENDINGDB_LOCK_DEBUGGING = Off
 # These directories are used to find various important files in the Mailman
 # installation.  PREFIX and EXEC_PREFIX are set by configure and should point
 # to the installation directory of the Mailman package.
-PYTHON          = '/2022Q2/bin/python2.7'
-PREFIX          = '/2022Q2/lib/mailman'
+PYTHON          = '/usr/pkg/bin/python2.7'
+PREFIX          = '/usr/pkg/lib/mailman'
 EXEC_PREFIX     = '${prefix}'
-VAR_PREFIX      = '/2022Q2/var/db/mailman'
+VAR_PREFIX      = '/var/db/mailman'
 
 # Work around a bogus autoconf 2.12 bug
 if EXEC_PREFIX == '${prefix}':
