@@ -90,6 +90,9 @@ def guess_extension(ctype, ext):
     if ctype.lower == 'application/octet-stream':
         # For this type, all[0] is '.obj'. '.bin' is better.
         return '.bin'
+    if ctype.lower == 'text/plain':
+        # For this type, all[0] is '.ksh'. '.txt' is better.
+        return '.txt'
     return all and all[0]
 
 
@@ -196,8 +199,11 @@ def process(mlist, msg, msgdata=None):
                 format = part.get_param('format')
                 delsp = part.get_param('delsp')
             # TK: if part is attached then check charset and scrub if none
-            if part.get('content-disposition') and \
-               not part.get_content_charset():
+            # MAS: Content-Disposition is not a good test for 'attached'.
+            # RFC 2183 sec. 2.10 allows Content-Disposition on the main body.
+            # Make it specifically 'attachment'.
+            if (part.get('content-disposition', '').lower() == 'attachment'
+                    and not part.get_content_charset()):
                 omask = os.umask(002)
                 try:
                     url = save_attachment(mlist, part, dir)
@@ -471,7 +477,8 @@ def save_attachment(mlist, msg, dir, filter_html=True):
             # guessed so attachments can't lie about their type.  Also, if the
             # filename /has/ no extension, then tack on the one we guessed.
             # The extension was removed from the name above.
-            filebase = filename
+            # Allow for extra and ext and keep it under 255 bytes.
+            filebase = filename[:240]
         # Now we're looking for a unique name for this file on the file
         # system.  If msgdir/filebase.ext isn't unique, we'll add a counter
         # after filebase, e.g. msgdir/filebase-cnt.ext
