@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2008 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -45,6 +45,8 @@ from Mailman.Logging.Syslog import syslog
 from Mailman import i18n
 
 _ = i18n._
+def D_(s):
+    return s
 
 # Request types requiring admin approval
 IGN = 0
@@ -101,7 +103,7 @@ class ListAdmin:
             # should we be as paranoid as for the config.pck file?  Should we
             # use pickle?
             tmpfile = self.__filename + '.tmp'
-            omask = os.umask(002)
+            omask = os.umask(007)
             try:
                 fp = open(tmpfile, 'w')
                 try:
@@ -194,7 +196,7 @@ class ListAdmin:
         else:
             ext = 'txt'
         filename = 'heldmsg-%s-%d.%s' % (self.internal_name(), id, ext)
-        omask = os.umask(002)
+        omask = os.umask(007)
         try:
             fp = open(os.path.join(mm_cfg.DATA_DIR, filename), 'w')
             try:
@@ -202,7 +204,7 @@ class ListAdmin:
                     cPickle.dump(msg, fp, 1)
                 else:
                     g = Generator(fp)
-                    g(msg, 1)
+                    g.flatten(msg, 1)
                 fp.flush()
                 os.fsync(fp.fileno())
             finally:
@@ -221,6 +223,8 @@ class ListAdmin:
         # an additional dictionary of message metadata
         #
         msgsubject = msg.get('subject', _('(no subject)'))
+        if not sender:
+            sender = _('<missing>')
         data = time.time(), sender, msgsubject, reason, filename, msgdata
         self.__db[id] = (HELDMSG, data)
         return id
@@ -241,7 +245,11 @@ class ListAdmin:
                 if e.errno <> errno.ENOENT: raise
                 return LOST
             try:
-                msg = cPickle.load(fp)
+                if path.endswith('.pck'):
+                    msg = cPickle.load(fp)
+                else:
+                    assert path.endswith('.txt'), '%s not .pck or .txt' % path
+                    msg = fp.read()
             finally:
                 fp.close()
             # Save the plain text to a .msg file, not a .pck file
@@ -250,8 +258,11 @@ class ListAdmin:
             outpath = head + '.msg'
             outfp = open(outpath, 'w')
             try:
-                g = Generator(outfp)
-                g(msg, 1)
+                if path.endswith('.pck'):
+                    g = Generator(outfp)
+                    g.flatten(msg, 1)
+                else:
+                    outfp.write(msg)
             finally:
                 outfp.close()
         # Now handle updates to the database
@@ -283,7 +294,8 @@ class ListAdmin:
             # message directly here can lead to a huge delay in web
             # turnaround.  Log the moderation and add a header.
             msg['X-Mailman-Approved-At'] = email.Utils.formatdate(localtime=1)
-            syslog('vette', 'held message approved, message-id: %s',
+            syslog('vette', '%s: held message approved, message-id: %s',
+                   self.internal_name(),
                    msg.get('message-id', 'n/a'))
             # Stick the message back in the incoming queue for further
             # processing.
@@ -292,9 +304,11 @@ class ListAdmin:
         elif value == mm_cfg.REJECT:
             # Rejected
             rejection = 'Refused'
+            lang = self.getMemberLanguage(sender)
+            subject = Utils.oneline(subject, Utils.GetCharSet(lang))
             self.__refuse(_('Posting of your message titled "%(subject)s"'),
                           sender, comment or _('[No reason given]'),
-                          lang=self.getMemberLanguage(sender))
+                          lang=lang)
         else:
             assert value == mm_cfg.DISCARD
             # Discarded
@@ -386,6 +400,7 @@ class ListAdmin:
                self.internal_name(), addr)
         # Possibly notify the administrator in default list language
         if self.admin_immed_notify:
+            i18n.set_language(self.preferred_language)
             realname = self.real_name
             subject = _(
                 'New subscription request to list %(realname)s from %(addr)s')
@@ -402,23 +417,32 @@ class ListAdmin:
             msg = Message.UserNotification(owneraddr, owneraddr, subject, text,
                                            self.preferred_language)
             msg.send(self, **{'tomoderators': 1})
+            # Restore the user's preferred language.
+            i18n.set_language(lang)
 
     def __handlesubscription(self, record, value, comment):
+        global _
         stime, addr, fullname, password, digest, lang = record
         if value == mm_cfg.DEFER:
             return DEFER
         elif value == mm_cfg.DISCARD:
-            pass
+            syslog('vette', '%s: discarded subscription request from %s',
+                   self.internal_name(), addr)
         elif value == mm_cfg.REJECT:
             self.__refuse(_('Subscription request'), addr,
                           comment or _('[No reason given]'),
                           lang=lang)
+            syslog('vette', """%s: rejected subscription request from %s
+\tReason: %s""", self.internal_name(), addr, comment or '[No reason given]')
         else:
             # subscribe
             assert value == mm_cfg.SUBSCRIBE
             try:
+                _ = D_
+                whence = _('via admin approval')
+                _ = i18n._
                 userdesc = UserDesc(addr, fullname, password, digest, lang)
-                self.ApprovedAddMember(userdesc, whence='via admin approval')
+                self.ApprovedAddMember(userdesc, whence=whence)
             except Errors.MMAlreadyAMember:
                 # User has already been subscribed, after sending the request
                 pass
@@ -460,9 +484,12 @@ class ListAdmin:
         if value == mm_cfg.DEFER:
             return DEFER
         elif value == mm_cfg.DISCARD:
-            pass
+            syslog('vette', '%s: discarded unsubscription request from %s',
+                   self.internal_name(), addr)
         elif value == mm_cfg.REJECT:
             self.__refuse(_('Unsubscription request'), addr, comment)
+            syslog('vette', """%s: rejected unsubscription request from %s
+\tReason: %s""", self.internal_name(), addr, comment or '[No reason given]')
         else:
             assert value == mm_cfg.UNSUBSCRIBE
             try:
