@@ -647,15 +647,15 @@ class DefinitionList(Container):
 #
 from mm_cfg import MAILMAN_URL
 PYTHON_URL  = 'http://www.python.org/'
-GNU_URL     = 'http://www.gnu.org/'
-OBSD_URL    = 'https://www.openbsd.org/'
 
 # The names of the image logo files.  These are concatentated onto
 # mm_cfg.IMAGE_LOGOS (not urljoined).
 DELIVERED_BY = 'mailman.jpg'
 PYTHON_POWERED = 'PythonPowered.png'
-GNU_HEAD = 'gnu-head-tiny.jpg'
-OBSD_HEAD = 'openbsd_pb.gif'
+
+from mm_cfg import IMAGE3_URL
+from mm_cfg import IMAGE3_IMG
+from mm_cfg import IMAGE3_ALT
 
 
 def MailmanLogo():
@@ -667,20 +667,17 @@ def MailmanLogo():
                  '<br>version %s' % (logo(DELIVERED_BY), mm_cfg.VERSION)
         pylink = '<img src="%s" alt="Python Powered" border=0>' % \
                  logo(PYTHON_POWERED)
-        gnulink = '<img src="%s" alt="GNU\'s Not Unix" border=0>' % \
-                  logo(GNU_HEAD)
-        obsdlink = '<img src="%s" alt="Powered by OpenBSD" border=0>' % \
-                  logo(OBSD_HEAD)
-        t.AddRow([mmlink, pylink, gnulink, obsdlink])
+        image3link = '<img src="%s" alt="%s" border=0>' % \
+                  (logo(IMAGE3_IMG), IMAGE3_ALT)
+        t.AddRow([mmlink, pylink, image3link])
     else:
         # use only textual links
         version = mm_cfg.VERSION
         mmlink = Link(MAILMAN_URL,
                       _('Delivered by Mailman<br>version %(version)s'))
         pylink = Link(PYTHON_URL, _('Python Powered'))
-        gnulink = Link(GNU_URL, _("Gnu's Not Unix"))
-        obsdlink = Link(OBSD_URL, 'Powered by OpenBSD')
-        t.AddRow([mmlink, pylink, gnulink, obsdlink])
+        image3link = Link(IMAGE3_URL, _(IMAGE3_ALT))
+        t.AddRow([mmlink, pylink, image3link])
     return t
 
 
