@@ -1,34 +1,28 @@
 Projekt polonizacji Mailmana rozpocz si w kwietniu 2001. 
 
-Oficjaln stron tumaczenia jest:
+Oficjaln stron tumaczenia jest: (strona praktycznie martwa)
 http://linux.gda.pl/pub/Mailman-pl/
 
-Lista dyskusyjna polskich tumaczy:
+Lista dyskusyjna polskich tumaczy: (lista praktycznie martwa)
 http://linux.gda.pl/mailman/listinfo/mailman-pl
 
-
-Do tej pory udzia w tumaczeniu wzieli (w kolejnoci chronologicznej
-przyczania si do projektu):
+Do tej pory w tumaczeniu pomagali (w kolejnoci chronologicznej):
 - Marcin Sochacki <wanted%linux.gda.pl>
 - Pawe Koodziejczyk <Pawel.Kolodziejczyk%comarch.pl>
 - Marcin Zaborowski <m.zaborowski%il.pw.edu.pl>
 - Wojciech Czarnowski <wojtekcz%rodos.com.pl>
 - Bartosz Sawicki <sawickib%iem.pw.edu.pl>
 - Robert wicki <rswieck1%elka.pw.edu.pl>
-
+- Pawe Goaszewski <blues%ds.pg.gda.pl>
+- Tomasz Toczyski <Tomasz.Toczyski%isep.pw.edu.pl>
 
 W chwili obecnej przetumaczone jest 99% interfejsu subskrybenta i okoo 
-85% interfejsu administracyjnego. Najbardziej aktualnej wersji szukajcie w
+95% interfejsu administracyjnego. Najbardziej aktualnej wersji szukajcie w
 oficjalnych CVS Mailmana. 
 
 Do zrobienia:
 - szlifowanie, polerowanie :)
-- skoczenie tumaczenia mailman.po (zostao jeszcze 297 msgid)
+- skoczenie tumaczenia mailman.po (zostao jeszcze 230 msgid)
 - przetumaczenie README.* dla rnych MTA
 
-Od jakiego roku nad tumaczeniem pracuj wlaciwie tylko sam, ale bardzo
-chtnie przyjm wszystkie formy pomocy.
-
-2003-12-30, Bartosz Sawicki <sawickib@iem.pw.edu.pl
-
-
+2005-10-11, Bartosz Sawicki <sawickib@iem.pw.edu.pl>
